(* Delphi Dialog
   Web Updater
   ===========
   Check Version on Web for Update (Thread based)
   Download file from URL and show Progress

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jul. 2008
   *)

unit WebUpdateDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, ComCtrls, IdComponent, IdHttp;

const
  defHttpProxyPort = 80;
    
type
  THttpProxyParams = record
    UseProxy       : boolean;
    Server,
    Username,
    Password       : string;
    Port           : integer;
    end;

  TUpdateStatus = (usNotAvail,usAvail,usCancel,usNoServer);
  TStatusEvent = procedure (var IsIdle : boolean) of object;

  TUpdateThread = class (TThread)
  private
    FCheckUrl,
    FVersion,
    FUrl,FName     : string;
    FLevel         : integer;
    IdHTTP         : TIdHTTP;
    FCanTerminate,
    FCancelled,
    FShowFault     : boolean;
    FUpdateStatus  : TUpdateStatus;
    FOnTermCheck   : TStatusEvent;
    procedure SetTermCheck (ACallBack : TStatusEvent);
    procedure DoTermCheck;
    procedure TermCheck;
  protected
    procedure Execute; override;
  public
    constructor Create (ACheckUrl,Version : string; AProxyPar : THttpProxyParams; ALevel : integer;
       AOnTermCheck : TStatusEvent; AOnTerminate :TNotifyEvent; AShowFault : boolean);
    destructor Destroy; override;
    procedure Cancel;
    property DownloadUrl : string read FUrl;
    property DownloadName : string read FName;
    property UpdateVersion : string read FVersion;
    property ShowFault : boolean read FShowFault;
    property UpdateStatus : TUpdateStatus read FUpdateStatus;
    property OnCheckTerminated : TStatusEvent read FOnTermCheck write SetTermCheck;
    property Terminated;
    end;

  TDownload = class(TForm)
    edURL: TLabeledEdit;
    edFilename: TLabeledEdit;
    pbDownload: TProgressBar;
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    nMax      : int64;
    procedure StartWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
    procedure ShowWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
  public
    { Public declarations }
    function Execute (const ATitle,URL,Filename : string; AProxyPar : THttpProxyParams) : boolean;
  end;

function IsNewerVersion (v1,v2 : string; Level : integer) : boolean;
procedure ResetProxy (var AProxyPar : THttpProxyParams);

var
  Download: TDownload;

implementation

{$R *.dfm}

uses JrUtils, GnuGetText;

{ ---------------------------------------------------------------- }
// True: v1<v2
function IsNewerVersion (v1,v2 : string; Level : integer) : boolean;
var
  n1,n2,i : cardinal;
begin
  n1:=0; i:=0;                               
  while (length(v1)>0) and (i<level) do begin
    n1:=100*n1+ReadNxtInt(v1,'.',0);
    inc(i);
    end;
  while (i<level) do begin
    n1:=100*n1;
    inc(i);
    end;
  n2:=0; i:=0;
  while (length(v2)>0) and (i<level) do begin
    n2:=100*n2+ReadNxtInt(v2,'.',0);
    inc(i);
    end;
  while (i<level) do begin
    n1:=100*n1;
    inc(i);
    end;
  Result:=n1<n2;
//  Result:=true;
  end;

// Reset proxy sttings
procedure ResetProxy (var AProxyPar : THttpProxyParams);
begin
  with AProxyPar do begin
    UseProxy:=false;
    Server:='';
    Username:='';
    Password:='';
    Port:=defHttpProxyPort;
    end;
  end;

{ ---------------------------------------------------------------- }
constructor TUpdateThread.Create (ACheckUrl,Version : string; AProxyPar : THttpProxyParams;
                                  ALevel : integer; AOnTermCheck : TStatusEvent;
                                  AOnTerminate :TNotifyEvent; AShowFault : boolean);
begin
  inherited Create (false);
  Priority:=tpLower;
//  FHandle:=CfHandle;
  FCheckUrl:=ACheckUrl;
  FVersion:=Version;
  FLevel:=ALevel;
  IdHTTP:=TIdHTTP.Create(nil);
  with IdHTTP do begin
    ReadTimeout:=10000;
    with AProxyPar do begin
      ProxyParams.BasicAuthentication:=UseProxy and (length(UserName)>0) and (length(Password)>0);
      if UseProxy then ProxyParams.ProxyServer:=Server
      else ProxyParams.ProxyServer:='';
      ProxyParams.ProxyPort:=Port;
      ProxyParams.ProxyUsername:=Username;
      ProxyParams.ProxyPassword:=Password;
      end;
    end;
  FreeOnTerminate:=true;
  FCanTerminate:=true;
  FCancelled:=false;
  FShowFault:=AShowFault;
  OnCheckTerminated:=AOnTermCheck;
  OnTerminate:=AOnTerminate;
  end;

destructor TUpdateThread.Destroy;
begin
  IdHTTP.Free;
  inherited;
  end;

procedure TUpdateThread.Cancel;
begin
  FCancelled:=true;
  end;

procedure TUpdateThread.SetTermCheck (ACallBack : TStatusEvent);
begin
  FOnTermCheck:=ACallBack;
  end;

procedure TUpdateThread.DoTermCheck;
begin
  if assigned(FOnTermCheck) then Synchronize(TermCheck)
  else FCanTerminate:=true;
  end;

procedure TUpdateThread.TermCheck;
begin
  FOnTermCheck(FCanTerminate);
  end;

procedure TUpdateThread.Execute;
var
  s,v           : string;
begin
  FUpdateStatus:=usNoServer;
  if length(FCheckUrl)>0 then begin
    try s:=IdHTTP.Get(FCheckUrl); except s:=''; end;
    if length(s)>0 then begin
      s:=CrLfToChar(s,'|');
      v:=ReadNxtStr(s,'|');
      if IsNewerVersion(FVersion,v,FLevel) then FUpdateStatus:=usAvail
      else FUpdateStatus:=usNotAvail;
      end;
    end;
  FVersion:=v;
  FUrl:=ReadNxtStr(s,'|');
  FName:=ReadNxtStr(s,'|');
//  if FHandle<>0 then PostMessage (FHandle,wmUpdateAvailable,ord(UpdateAvail),0);
  if FCancelled then FUpdateStatus:=usCancel;
//  FUpdateStatus:=usAvail; // zum Testen
  repeat
    Sleep(200);
    DoTermCheck;
    until FCanTerminate;
  Terminate;
  end;

{ ---------------------------------------------------------------- }
procedure TDownload.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;


procedure TDownload.StartWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
begin
  if AWorkMode=wmRead then nMax:=AWorkCount;
  pbDownload.Position:=0;
  Application.ProcessMessages;
  end;

procedure TDownload.ShowWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
begin
  if (AWorkMode=wmRead) and (nMax>0) then
    pbDownload.Position:=round((AWorkCount/nMax)*100);
  Application.ProcessMessages;
  end;

function TDownload.Execute (const ATitle,URL,Filename : string; AProxyPar : THttpProxyParams) : boolean;
var
  fs      : TFileStream;
  IdHTTP  : TIdHTTP;
begin
  Caption:=ATitle;
  edURL.Text:=URL;
  edFilename.Text:=Filename;
  IdHTTP:=TIdHTTP.Create(Application);
  with IdHTTP do begin
    with AProxyPar do begin
      ProxyParams.BasicAuthentication:=UseProxy and (length(UserName)>0) and (length(Password)>0);
      if UseProxy then ProxyParams.ProxyServer:=Server
      else ProxyParams.ProxyServer:='';
      ProxyParams.ProxyPort:=Port;
      ProxyParams.ProxyUsername:=Username;
      ProxyParams.ProxyPassword:=Password;
      end;
    OnWorkBegin:=StartWork;
    OnWork:=ShowWork;
    end;
  Result:=false;
  Show;
  BringToFront;
  try
    fs:=TFileStream.Create(Filename,fmCreate);
    try
      IdHTTP.Get(Url,fs);
      Result:=true;
    except
      end;
  finally
    fs.Free;
    end;
  IdHTTP.Free;
  Close;
  end;

end.
